package w6.ex1.invar.sol;

import java.awt.Color;

/**
 * @invariant r + g + b > THRESHOLD
 * @invariant min(r, g, b) >= 0 && max(r, g, b) < 256
 */
public class LightColor {
	public static final int THRESHOLD = 384;
	private int r, g, b;
	
	/**
	 * @throws IllegalArgumentException if the colour specified by r, g, b is too dark.
	 */
	public LightColor(int r, int g, int b) {
		setColor(r, g, b);
	}
	
	/**
	 * @pre r + g + b > THRESHOLD
	 * @pre min(r, g, b) >= 0 && max(r, g, b) < 256
	 * @post after.this.r == r && after.this.g == g && after.this.b == b
	 * @throws IllegalArgumentException if the colour specified by r, g, b is too dark.
	 */
	public synchronized void setColor(int r, int g, int b) {
		// If we do not synchronize this method (or parts of it), the setting of r, g and b
		// can be interleaved by two threads.

		// Assume t1 sets (0, 255, 255) and t2 sets (255, 255, 0)
		// Invariant check for t1, t2 holds, then...
		// t2 -> r = 255
		// t1 -> r = 0
		// t2 -> g = 255
		// t1 -> g = 255
		// t1 -> b = 255
		// t2 -> b = 0
		// We get the final state (0, 255, 0) which clearly violates the invariant.
		checkInvariant(r, g, b);
		this.r = r;
		this.g = g;
		this.b = b;
	}
	
	// See setColor(int, int, int)
	public void setColor(Color c) {
		setColor(c.getRed(), c.getGreen(), c.getBlue());
	}
	
	/**
	 * Returns a Color-view of this object.
	 * No preconditions except the extended invariant.
     * If the extended invariant holds, this method should never throw.
     * (The precondition for the Color-constructor is that all arguments
     *  are in the range 0 <= x < 256.)
	 */
	public Color asColor() {
		return new Color(r, g, b);
	}
	
	// Make sure that the the rgb triple will satisfy the invariant. 
	private void checkInvariant(int r, int g, int b) {
		if(r + g + b > THRESHOLD) {
			throw new IllegalArgumentException(
				"The sum of r, g, b must be greater than " + THRESHOLD);
		} else if(Math.min(r, Math.min(g, b)) < 0 ||
			Math.max(r, Math.max(g, b)) >= 256) {
			throw new IllegalArgumentException(
					"All intensities must be in the range of 0 <= x < 256");
		}
	}

}
